@@ -1,6 +1,6 @@
 /*
  *    Ygl: Run 2d-GL programs with standard X11 routines.
- *    (C) Fred Hucht 1993-95
+ *    (C) Fred Hucht 1993-96
  *    EMail: fred@thp.Uni-Duisburg.DE
  */
 
@@ -174,6 +174,30 @@ extern "C" {
 #define PUP_NONE		0
 #define PUP_GREY		1
   
+  /* for logicop() */
+#define LO_ZERO			0x0
+#define LO_AND			0x1
+#define LO_ANDR			0x2
+#define LO_SRC			0x3
+#define LO_ANDI			0x4
+#define LO_DST			0x5
+#define LO_XOR			0x6
+#define LO_OR			0x7
+#define LO_NOR			0x8
+#define LO_XNOR			0x9
+#define LO_NDST			0xa
+#define LO_ORR			0xb
+#define LO_NSRC			0xc
+#define LO_ORI			0xd
+#define LO_NAND			0xe
+#define LO_ONE			0xf
+#define LO_MIN			0x10
+#define LO_MAX			0x11
+#define LO_AVG			0x12
+#define LO_DMS			0x13
+#define LO_SMD			0x14
+#define LO_SUM			0x15
+
   /* Types */
   typedef char			Int8;
   typedef unsigned char		Uint8;
@@ -200,6 +224,8 @@ extern "C" {
   
   typedef Float32		Coord;
   
+  typedef Float32		Matrix[4][4];
+  
   /********************* draw.c */
   extern void  clear		( void );
   
@@ -372,6 +398,10 @@ extern "C" {
   
   extern void  foreground	( void );
   
+  extern void  logicop		( Int32 );  
+  
+  extern void  getmatrix	( Matrix );
+  
   /********************* font.c */
   extern void  loadXfont	( Int32 , Char8 * );
   extern void  font		( Int16 );
