@@ -1,16 +1,17 @@
 /*
  *    Ygl: Run 2d-GL programs with standard X11 routines.
- *    (C) Fred Hucht 1993-95
+ *    (C) Fred Hucht 1993-96
  *    EMail: fred@thp.Uni-Duisburg.DE
  */
 
 #include "header.h"
 
 static int   rect_read_errh(Display *, XErrorEvent *);
-static Int32 rect_read(Screencoord, Screencoord, Screencoord, Screencoord, int, void *);
-static void rect_write(Screencoord, Screencoord, Screencoord, Screencoord, int, void *);
+static Int32 rect_read(const char*, Screencoord, Screencoord, Screencoord, Screencoord, int, void *);
+static void rect_write(const char*, Screencoord, Screencoord, Screencoord, Screencoord, int, void *);
 static Ulong emuColorsInv(Ulong x);
 static void create_dither(int r, int g, int b);
+static void set_color(const char *caller, int r, int g, int b);
 
 #define YGL_COLORSINV(x) (Ygl.PCM ? CMapSize-1-(x) : Ygl.EmulateCmap ? emuColorsInv(x) : Ygl.ColorsInv[x])
 
@@ -22,9 +23,11 @@ static Ulong emuColorsInv(Ulong x) {
 
 void mapcolor(Colorindex ind, Int16 r, Int16 g, Int16 b) {
   XColor xc;
-  I("mapcolor");
+  const char * MyName = "mapcolor";
+  I(MyName);
+  if(W->rgb) { Yprintf(MyName, "not in CMap mode.\n"); return; }
   if(ind >= CMapSize) {
-    fprintf(stderr, "Ygl: mapcolor: can't map cell %d (must be between 0 and %d).\n", ind, CMapSize-1);
+    Yprintf(MyName, "can't map cell %d (must be between 0 and %d).\n", ind, CMapSize-1);
     return;
   }
   xc.red   = r << 8;
@@ -38,7 +41,7 @@ void mapcolor(Colorindex ind, Int16 r, Int16 g, Int16 b) {
       if(!Ygl.EmulateCmap)
 	Ygl.ColorsInv[xc.pixel] = ind;
     } else {
-      fprintf(stderr, "Ygl: mapcolor: can't allocate color ( %d, %d, %d) for cell %d.\n", r, g, b, ind);
+      Yprintf(MyName, "can't allocate color ( %d, %d, %d) for cell %d.\n", r, g, b, ind);
     }
   }
   else { /* use private colormap */
@@ -51,7 +54,7 @@ void mapcolor(Colorindex ind, Int16 r, Int16 g, Int16 b) {
 #endif
   if(!Ygl.GC) { /* if GC list in CMap windows */
     int i;
-    YglWindow *w = Ygl.Windows + 1; /* first window */
+    YglWindow *w = &Ygl.Windows[1]; /* first window */
     for(i = 1; i < Ygl.NextWindow; i++, w++) {
       if(w->main != 0 && !w->rgb) {
 	XSetForeground(D, w->gclist[ind], ind);
@@ -64,29 +67,31 @@ void mapcolor(Colorindex ind, Int16 r, Int16 g, Int16 b) {
 static const char *DI[] = { 	/* index vector for dither matrices */
   "\000", 			/* DI[sz][c] >> 3 : x-coordinate */
   "\000", 			/* DI[sz][c] &  7 : y-coordinate */
-  
-  "\000\011"
-  "\001\010", 					/* 2x2 */
-    
-  "\000\011\012\020\001\010\022\021\002", 	/* 3x3 */
-    
-  "\000\022\002\020"
-  "\011\033\013\031"
-  "\010\032\012\030"
-  "\021\003\023\001", 				/* 4x4 */
-    
-  "\000\012\024\031\043"
-  "\010\022\034\041\003"
-  "\020\032\044\001\013"
-  "\030\042\004\011\023"
-  "\040\002\014\021\033", 			/* 5x5 */
-						/* +-0--1--2--3--4--5*/
-  "\000\033\002\035\004\031" 			/*0| 0 18 16 34  9 27*/
-  "\011\044\013\040\015\042" 			/*1|33  6 24  5 23 15*/
-  "\022\055\024\051\020\053" 			/*2| 2 20 12 30 11 29*/
-  "\010\043\012\045\014\041" 			/*3|35  8 26  1 19 17*/
-  "\021\054\023\050\025\052" 			/*4| 4 22 14 32  7 25*/
-  "\032\005\034\001\030\003" 			/*5|31 10 28  3 21 13*/
+  				/* 2x2  ***+-0--1-*/
+  "\000\011"				/*0| 0  3 */
+  "\001\010", 				/*1| 2  1 */
+    				/* 3x3  ***+-0--1--2-*/
+  "\000\011\012"			/*0| 0  5  3 */
+  "\020\001\010"			/*1| 4  1  7 */
+  "\022\021\002", 			/*2| 8  2  6 */
+    				/* 4x4  ***+-0--1--2--3-*/
+  "\000\022\002\020"			/*0| 0  8  3 11 */
+  "\011\033\013\031"			/*1|12  4 15  7 */
+  "\010\032\012\030"			/*2| 2 10  1  9 */
+  "\021\003\023\001", 			/*3|14  6 13  5 */
+    				/* 5x5  ***+-0--1--2--3--4-*/
+  "\000\012\024\031\043"		/*0| 0  5 10 15 20 */
+  "\010\022\034\041\003"		/*1|13 18 23  3  8 */
+  "\020\032\044\001\013"		/*2|21  1  6 11 16 */
+  "\030\042\004\011\023"		/*3| 9 14 19 24  4 */
+  "\040\002\014\021\033", 		/*4|17 22  2  7 12 */
+				/* 6x6  ***+-0--1--2--3--4--5-*/
+  "\000\033\002\035\004\031" 		/*0| 0 18 16 34  9 27 */
+  "\011\044\013\040\015\042" 		/*1|33  6 24  5 23 15 */
+  "\022\055\024\051\020\053" 		/*2| 2 20 12 30 11 29 */
+  "\010\043\012\045\014\041" 		/*3|35  8 26  1 19 17 */
+  "\021\054\023\050\025\052" 		/*4| 4 22 14 32  7 25 */
+  "\032\005\034\001\030\003" 		/*5|31 10 28  3 21 13 */
 };
 
 static void create_dither(int r, int g, int b) {
@@ -128,9 +133,9 @@ static void create_dither(int r, int g, int b) {
       mask = GCFillStyle | GCForeground;
       values.fill_style = FillSolid;
       values.foreground =
-	(    (rx << Ygl.rs) & Ygl.RMask) +
-	  (  (gx << Ygl.gs) & Ygl.GMask) +
-	    ((bx << Ygl.bs) & Ygl.BMask);
+	((rx << Ygl.rs) & Ygl.RMask) +
+	((gx << Ygl.gs) & Ygl.GMask) +
+	((bx << Ygl.bs) & Ygl.BMask);
     } else {
       /* Dither. */
       mask = GCTile | GCFillStyle;
@@ -158,18 +163,23 @@ static void create_dither(int r, int g, int b) {
 
 /* converts 24 bit RGB values ( 0xBBGGRR ) to value appropiate for RGBVisual */
 #define RGB24_TO_RGBVisual(rgb24) ( \
-		((((rgb24) >> (Ygl.rb +  0)) << Ygl.rs) & Ygl.RMask) \
-	      + ((((rgb24) >> (Ygl.gb +  8)) << Ygl.gs) & Ygl.GMask) \
-	      + ((((rgb24) >> (Ygl.bb + 16)) << Ygl.bs) & Ygl.BMask) )
+	((((rgb24) >> (Ygl.rb +  0)) << Ygl.rs) & Ygl.RMask) \
+      + ((((rgb24) >> (Ygl.gb +  8)) << Ygl.gs) & Ygl.GMask) \
+      + ((((rgb24) >> (Ygl.bb + 16)) << Ygl.bs) & Ygl.BMask) )
+
+#define RGBVisual_TO_RGB24(pixel) ( \
+	((((pixel) & Ygl.RMask) >> Ygl.rs) << (Ygl.rb +  0)) \
+      + ((((pixel) & Ygl.GMask) >> Ygl.gs) << (Ygl.gb +  8)) \
+      + ((((pixel) & Ygl.BMask) >> Ygl.bs) << (Ygl.bb + 16)) )
 
 #define RGB_TO_RGBVisual(r,g,b) ( \
-		((((r) >> Ygl.rb) << Ygl.rs) & Ygl.RMask) \
-	      + ((((g) >> Ygl.gb) << Ygl.gs) & Ygl.GMask) \
-	      + ((((b) >> Ygl.bb) << Ygl.bs) & Ygl.BMask) )
+	((((r) >> Ygl.rb) << Ygl.rs) & Ygl.RMask) \
+      + ((((g) >> Ygl.gb) << Ygl.gs) & Ygl.GMask) \
+      + ((((b) >> Ygl.bb) << Ygl.bs) & Ygl.BMask) )
 
-void RGBcolor (Int16 r, Int16 g, Int16 b) {
-  I("RGBcolor");
-  if(!W->rgb) { fprintf(stderr, "Ygl: RGBcolor: not in RGB mode.\n"); return; }
+static void set_color(const char *caller, int r, int g, int b) {
+  I(caller);
+  if(!W->rgb) { Yprintf(caller, "not in RGB mode.\n"); return; }
   if(W->pm) {
     create_dither(r, g, b);
   } else {
@@ -177,76 +187,57 @@ void RGBcolor (Int16 r, Int16 g, Int16 b) {
   }
 }
 
+void RGBcolor(Int16 r, Int16 g, Int16 b) {
+  const char * MyName = "RGBcolor";
+  set_color(MyName, r, g, b);
+}
+
 void cpack(Uint32 rgb) {
-  I("cpack");
-  if(!W->rgb) { fprintf(stderr, "Ygl: cpack: not in RGB mode.\n");return; }
-  if(W->pm) {
-    create_dither((rgb >>  0) & 0xFF,
-		  (rgb >>  8) & 0xFF,
-		  (rgb >> 16) & 0xFF);
-  } else {
-    XSetForeground(D, W->gc, RGB24_TO_RGBVisual(rgb));
-  }
+  set_color("cpack", 
+	    (rgb >>  0) & 0xFF, (rgb >>  8) & 0xFF, (rgb >> 16) & 0xFF);
 }
 
 void c3s(Int16 c[3]) {
-  I("c3s");
-  if(!W->rgb) { fprintf(stderr, "Ygl: c3s: not in RGB mode.\n"); return; }
-  if(W->pm) {
-    create_dither(c[0], c[1], c[2]);
-  } else {
-    XSetForeground(D, W->gc, RGB_TO_RGBVisual(c[0], c[1], c[2]));
-  }
+  set_color("c3s", c[0], c[1], c[2]);
 }
 
 void c3i(Int32 c[3]) {
-  I("c3i");
-  if(!W->rgb) { fprintf(stderr, "Ygl: c3i: not in RGB mode.\n"); return; }
-  if(W->pm) {
-    create_dither(c[0], c[1], c[2]);
-  } else {
-    XSetForeground(D, W->gc, RGB_TO_RGBVisual(c[0], c[1], c[2]));
-  }
+  set_color("c3i", c[0], c[1], c[2]);
 }
 
-void c3f(float c[3]) {
-  I("c3f");
-  if(!W->rgb) { fprintf(stderr, "Ygl: c3f: not in RGB mode.\n"); return; }
-  if(W->pm) {
-    create_dither((int)c[0]*255.99, (int)c[1]*255.99, (int)c[2]*255.99);
-  } else {
-    XSetForeground(D, W->gc, RGB_TO_RGBVisual((Uint32)(c[0]*255.99),
-					      (Uint32)(c[1]*255.99),
-					      (Uint32)(c[2]*255.99)));
-  }
+void c3f(Float32 c[3]) {
+  float f = 255.9999999;
+  set_color("c3f", (int)(f * c[0]), (int)(f * c[1]), (int)(f * c[2]));
 }
 
 #if 0 /* This version is veeeryyy slooowwww !!! */
 void RGBcolor (Int16 r, Int16 g, Int16 b) {
   XColor xc;
-  I("RGBcolor");
-  if(!W->rgb) {
-    fprintf(stderr, "Ygl: RGBcolor: not in RGB mode.\n");
-    return;
-  }
+  const char * MyName = "RGBcolor";
+  I(MyName);
+  if(!W->rgb) { Yprintf(MyName, "not in RGB mode.\n"); return; }
   xc.red   = r << 8;
   xc.green = g << 8;
   xc.blue  = b << 8;
   if(XAllocColor(D, Ygl.RCmap, &xc))
     XSetForeground(D, W->gc, xc.pixel);
   else
-    fprintf(stderr, "Ygl: RGBcolor: can't allocate color ( %d, %d, %d).\n", r, g, b);
+    Yprintf(MyName, "can't allocate color ( %d, %d, %d).\n", r, g, b);
 }
 #endif
 
 Int32 getcolor(void) {
-  I("getcolor");
+  const char * MyName = "getcolor";
+  I(MyName);
+  if(W->rgb) { Yprintf(MyName, "not in CMap mode.\n"); return 0; }
   return W->color;
 }
 
 void getmcolor (Colorindex ind, Int16 *r, Int16 *g, Int16 *b) {
   XColor xc;
-  I("getmcolor");
+  const char * MyName = "getmcolor";
+  I(MyName);
+  if(W->rgb) { Yprintf(MyName, "not in CMap mode.\n"); return; }
   xc.pixel = YGL_COLORS(ind);
   XQueryColor(D, Ygl.CCmap, &xc);
   *r = xc.red   >> 8;
@@ -258,15 +249,20 @@ void getmcolors (Colorindex ind1, Colorindex ind2, Int16 *r, Int16 *g, Int16 *b)
   XColor *xc;
   Colorindex i;
   int n = ind2 - ind1 + 1;
-  I("getmcolors");
-  
+  const char * MyName = "getmcolors";
+  I(MyName);
+  if(W->rgb) {
+    Yprintf(MyName, "not in CMap mode.\n");
+    return;
+  }
+
   if(n < 0) {
-    fprintf(stderr, "Ygl: getmcolors: 2nd argument < 1st argument.\n");
+    Yprintf(MyName, "2nd argument < 1st argument.\n");
     return;
   }
   
   if(NULL == (xc = (XColor*) malloc(n * sizeof(XColor)))) {
-    fprintf(stderr, "Ygl: getmcolors: can't allocate memory.\n");
+    Yprintf(MyName, "can't allocate memory.\n");
     exit(1);
   }
 
@@ -282,7 +278,12 @@ void getmcolors (Colorindex ind1, Colorindex ind2, Int16 *r, Int16 *g, Int16 *b)
 
 void gRGBcolor(Int16 *r, Int16 *g, Int16 *b) {
   XGCValues ret;
-  I("gRGBcolor");
+  const char * MyName = "gRGBcolor";
+  I(MyName);
+  if(!W->rgb) {
+    Yprintf(MyName, "not in RGB mode.\n");
+    return;
+  }
   if(W->pm) {
     *r = W->red;
     *g = W->green;
@@ -293,7 +294,7 @@ void gRGBcolor(Int16 *r, Int16 *g, Int16 *b) {
       *g = ((ret.foreground & Ygl.GMask) >> Ygl.gs) << Ygl.gb;
       *b = ((ret.foreground & Ygl.BMask) >> Ygl.bs) << Ygl.bb;
     } else {
-      fprintf(stderr, "Ygl: gRGBcolor: cannot get GCValues.\n");
+      Yprintf(MyName, "cannot get GCValues.\n");
     }
   }
 #ifdef DEBUG
@@ -302,9 +303,12 @@ void gRGBcolor(Int16 *r, Int16 *g, Int16 *b) {
 }
 
 void color(Colorindex ind) {
-  I("color");
-  if(ind >= CMapSize)
-    fprintf(stderr, "Ygl: color: invalid color %d (must be between 0 and %d).\n", ind, CMapSize-1);
+  const char * MyName = "color";
+  I(MyName);
+  if(W->rgb)
+    Yprintf(MyName, "not in CMap mode.\n");
+  else if(ind >= CMapSize)
+    Yprintf(MyName, "invalid color %d (must be between 0 and %d).\n", ind, CMapSize-1);
   else {
     W->color = ind;
     if(Ygl.GC) /* single GC */
@@ -316,9 +320,12 @@ void color(Colorindex ind) {
 
 #ifndef YGL_PREFIX
 void Xcolor(Colorindex ind) {
-  I("Xcolor");
-  if(ind >= CMapSize)
-    fprintf(stderr, "Ygl: Xcolor: invalid color %d (must be between 0 and %d).\n", ind, CMapSize-1);
+  const char * MyName = "Xcolor";
+  I(MyName);
+  if(W->rgb)
+    Yprintf(MyName, "not in CMap mode.\n");
+  else if(ind >= CMapSize)
+    Yprintf(MyName, "invalid color %d (must be between 0 and %d).\n", ind, CMapSize-1);
   else {
     W->color = YGL_COLORSINV(ind);
     if(Ygl.GC) /* single GC */
@@ -330,12 +337,17 @@ void Xcolor(Colorindex ind) {
 
 void getmXcolor (Colorindex ind, Int16 *r, Int16 *g, Int16 *b) {
   XColor xc;
-  I("getmXcolor");
-  xc.pixel = ind;
-  XQueryColor(D, Ygl.CCmap, &xc);
-  *r = xc.red   >> 8;
-  *g = xc.green >> 8;
-  *b = xc.blue  >> 8;
+  const char * MyName = "getmXcolor";
+  I(MyName);
+  if(W->rgb)
+    Yprintf(MyName, "not in CMap mode.\n");
+  else {
+    xc.pixel = ind;
+    XQueryColor(D, Ygl.CCmap, &xc);
+    *r = xc.red   >> 8;
+    *g = xc.green >> 8;
+    *b = xc.blue  >> 8;
+  }
 }
 #endif
 
@@ -343,6 +355,8 @@ void getmXcolor (Colorindex ind, Int16 *r, Int16 *g, Int16 *b) {
 static int rect_read_front = True;
 
 void readsource(Int32 source) {
+  const char * MyName = "readsource";
+  I(MyName);
   switch(source) {
   case SRC_AUTO: 
   case SRC_BACK: 
@@ -352,20 +366,19 @@ void readsource(Int32 source) {
     rect_read_front = True;
     break;
   default:
-    fprintf(stderr, "Ygl: readsource: unknown mode %d.", source);
+    Yprintf(MyName, "unknown mode %d.\n", source);
     break;
   }
 }
 
 static int rect_read_err;
-static const char rect_first[] = "?c ?l"; /* first char for errors */
-
 static int rect_read_errh(Display *dpy, XErrorEvent *error) {
   rect_read_err = error->error_code;
   return 0;
 }
 
-static Int32 rect_read(Screencoord x1, Screencoord y1,
+static Int32 rect_read(const char *caller,
+		       Screencoord x1, Screencoord y1,
 		       Screencoord x2, Screencoord y2,
 		       int size, void *data) {
   XImage *XI;
@@ -373,15 +386,19 @@ static Int32 rect_read(Screencoord x1, Screencoord y1,
   register Int32 *data32 = (Int32*) data;
   register Int16 *data16 = (Int16*) data;
   register Uint8 *data8  = (Uint8*) data;
-  Screencoord width  = x2-x1+1;
-  Screencoord height = y2-y1+1;
+  Screencoord width  = x2 - x1 + 1;
+  Screencoord height = y2 - y1 + 1;
   int (*old_handler)(Display*, XErrorEvent*);
   
   rect_read_err = 0; /* reset */
   old_handler = XSetErrorHandler(rect_read_errh);
   
   XI = XGetImage(D, 
+#ifdef RGBWIN
 		 rect_read_front ? W->win : W->draw,
+#else
+		 rect_read_front ? W->main : W->draw,
+#endif
 		 x1, W->ym - 1 - y2,
 		 width, height,
 		 (1 << YglDepth())-1,
@@ -395,7 +412,7 @@ static Int32 rect_read(Screencoord x1, Screencoord y1,
 	  width, height, XI->width, XI->height, rect_read_err);
 #endif
   if(rect_read_err == BadMatch) {
-    fprintf(stderr, "Ygl: %crectread: window not completely visible.\n", rect_first[size]);
+    Yprintf(caller, "window not completely visible.\n");
     return 0;
   }
   
@@ -410,9 +427,7 @@ static Int32 rect_read(Screencoord x1, Screencoord y1,
       pixel = XGetPixel(XI, x, y);
       
       if(W->rgb) {
-	val = (((pixel & Ygl.RMask) >> Ygl.rs) << (Ygl.rb +  0))
-	    + (((pixel & Ygl.GMask) >> Ygl.gs) << (Ygl.gb +  8))
-	    + (((pixel & Ygl.BMask) >> Ygl.bs) << (Ygl.bb + 16));
+	val = RGBVisual_TO_RGB24(pixel);
       } else {
 	val = YGL_COLORSINV(pixel);
       }
@@ -439,7 +454,8 @@ static Int32 rect_read(Screencoord x1, Screencoord y1,
   return x;
 }
 
-static void rect_write(Screencoord x1, Screencoord y1,
+static void rect_write(const char *caller,
+		       Screencoord x1, Screencoord y1,
 		       Screencoord x2, Screencoord y2,
 		       int size, void *data) {
   XImage *XI;
@@ -447,8 +463,8 @@ static void rect_write(Screencoord x1, Screencoord y1,
   register Int32 *data32 = (Int32*) data;
   register Int16 *data16 = (Int16*) data;
   register Uint8 *data8  = (Uint8*) data;
-  Screencoord width  = x2-x1+1;
-  Screencoord height = y2-y1+1;
+  Screencoord width  = x2 - x1 + 1;
+  Screencoord height = y2 - y1 + 1;
   
   XI = XCreateImage(D,
 		    YglVisual(),
@@ -462,7 +478,7 @@ static void rect_write(Screencoord x1, Screencoord y1,
 		    0);		/* BytesPerLine */
 
   if(NULL == (XI->data = (char*) malloc(XI->bytes_per_line * height))) {
-    fprintf(stderr, "Ygl: %crectwrite: can't allocate memory.\n",  rect_first[size]);
+    Yprintf(caller, "can't allocate memory.\n");
     exit(1);
   }
   
@@ -479,9 +495,9 @@ static void rect_write(Screencoord x1, Screencoord y1,
       XPutPixel(XI, x, y, pixel); 
       /* *(d + y * XI->bytes_per_scanline + x) = pixel; */
     } else { /* CMap */
-      if((val < 0) || (val >= CMapSize)) {
-	fprintf(stderr, "Ygl: %crectwrite: invalid color %d at position (%d,%d) in cmode (must be between 0 and %d).\n",
-		rect_first[size], val, x, y, CMapSize-1);
+      if(val < 0 || val >= CMapSize) {
+	Yprintf(caller, "invalid color %d at position (%d,%d) in CMap mode (must be between 0 and %d).\n",
+		val, x, y, CMapSize-1);
       } else {
 	XPutPixel(XI, x, y, YGL_COLORS(val));
       }
@@ -496,21 +512,23 @@ static void rect_write(Screencoord x1, Screencoord y1,
   F;
 }
 
-Int32 lrectread(Screencoord x1, Screencoord y1,	Screencoord x2, Screencoord y2,	Int32 *data) { I("lrectread"); return rect_read(x1,y1,x2,y2, 4, data);}
-Int32  rectread(Screencoord x1, Screencoord y1,	Screencoord x2, Screencoord y2,	Int16 *data) { I( "rectread"); return rect_read(x1,y1,x2,y2, 2, data);}
-Int32 crectread(Screencoord x1, Screencoord y1,	Screencoord x2, Screencoord y2,	Uint8 *data) { I("crectread"); return rect_read(x1,y1,x2,y2, 1, data);}
+Int32 lrectread(Screencoord x1, Screencoord y1,	Screencoord x2, Screencoord y2,	Int32 *data) { const char * MyName = "lrectread"; I(MyName); return rect_read(MyName, x1,y1,x2,y2, 4, data);}
+Int32  rectread(Screencoord x1, Screencoord y1,	Screencoord x2, Screencoord y2,	Int16 *data) { const char * MyName = "rectread" ; I(MyName); return rect_read(MyName, x1,y1,x2,y2, 2, data);}
+Int32 crectread(Screencoord x1, Screencoord y1,	Screencoord x2, Screencoord y2,	Uint8 *data) { const char * MyName = "crectread"; I(MyName); return rect_read(MyName, x1,y1,x2,y2, 1, data);}
 
-void lrectwrite(Screencoord x1, Screencoord y1, Screencoord x2, Screencoord y2, Int32 *data) { I("lrectwrite"); rect_write(x1,y1,x2,y2, 4, data);}
-void  rectwrite(Screencoord x1, Screencoord y1,	Screencoord x2, Screencoord y2,	Int16 *data) { I( "rectwrite"); rect_write(x1,y1,x2,y2, 2, data);}
-void crectwrite(Screencoord x1, Screencoord y1,	Screencoord x2, Screencoord y2,	Uint8 *data) { I("crectwrite"); rect_write(x1,y1,x2,y2, 1, data);}
+void lrectwrite(Screencoord x1, Screencoord y1, Screencoord x2, Screencoord y2, Int32 *data) { const char * MyName = "lrectwrite"; I(MyName); rect_write(MyName, x1,y1,x2,y2, 4, data);}
+void  rectwrite(Screencoord x1, Screencoord y1,	Screencoord x2, Screencoord y2,	Int16 *data) { const char * MyName = "rectwrite" ; I(MyName); rect_write(MyName, x1,y1,x2,y2, 2, data);}
+void crectwrite(Screencoord x1, Screencoord y1,	Screencoord x2, Screencoord y2,	Uint8 *data) { const char * MyName = "crectwrite"; I(MyName); rect_write(MyName, x1,y1,x2,y2, 1, data);}
 
 void rectcopy(Screencoord x1, Screencoord y1,
 	      Screencoord x2, Screencoord y2,
 	      Screencoord xn, Screencoord yn) {
-  I("rectcopy");
+  const char * MyName = "rectcopy";
+  I(MyName);
   XCopyArea(D, W->draw, W->draw, W->gc,
 	    x1, W->ym - 1 - y2,
-	    x2-x1+1, y2-y1+1,
+	    x2 - x1 + 1,
+	    y2 - y1 + 1,
 	    xn, W->ym - 1 - yn - (y2-y1));
   F;
 }
@@ -518,47 +536,50 @@ void rectcopy(Screencoord x1, Screencoord y1,
 Int32 readpixels(Int16 n, Colorindex data[]) {
   Screencoord x, y;
   Int32 r;
-  I("readpixels");
+  const char * MyName = "readpixels";
+  I(MyName);
   if(W->rgb) {
-    fprintf(stderr, "Ygl: readpixels: not in cmap mode.\n");
+    Yprintf(MyName, "not in CMap mode.\n");
     exit(1);
   }
   x = X(W->xc);
   y = Y(W->yc);
-  r = rect_read(x, y, x + n, y + 1, 2, data);
+  r = rect_read(MyName, x, y, x + n, y + 1, 2, data);
   W->xc += n / W->xf;
   return r;
 }
 
 void writepixels(Int16 n, Colorindex data[]) {
   Screencoord x, y;
-  I("writepixels");
+  const char * MyName = "writepixels";
+  I(MyName);
   if(W->rgb) {
-    fprintf(stderr, "Ygl: writepixels: not in cmap mode.\n");
+    Yprintf(MyName, "not in CMap mode.\n");
     exit(1);
   }
   x = X(W->xc);
   y = Y(W->yc);
-  rect_write(x, y, x + n, y + 1, 2, data);
+  rect_write(MyName, x, y, x + n, y + 1, 2, data);
   W->xc += n / W->xf;
 }
 
 Int32 readRGB(Int16 n, RGBvalue r[], RGBvalue g[], RGBvalue b[]) {
   Screencoord x, y;
   Int32 ret, *data, i;
-  I("readRGB");
+  const char * MyName = "readRGB";
+  I(MyName);
   if(!W->rgb) {
-    fprintf(stderr, "Ygl: readRGB: not in RGB mode.\n");
+    Yprintf(MyName, "not in RGB mode.\n");
     exit(1);
   }
   x = X(W->xc);
   y = Y(W->yc);
   W->xc += n / W->xf;
   if(NULL == (data = (Int32*)malloc(n * sizeof(Int32)))) {
-    fprintf(stderr, "Ygl: readRGB: out of memory.\n");
+    Yprintf(MyName, "out of memory.\n");
     exit(1);
   }
-  ret = rect_read(x, y, x + n, y + 1, 4, data);
+  ret = rect_read(MyName, x, y, x + n, y + 1, 4, data);
   for(i = 0; i < n; i++) {
     r[i] = (data[i] >>  0) & 0xFF;
     g[i] = (data[i] >>  8) & 0xFF;
@@ -571,21 +592,22 @@ Int32 readRGB(Int16 n, RGBvalue r[], RGBvalue g[], RGBvalue b[]) {
 void writeRGB(Int16 n, RGBvalue r[], RGBvalue g[], RGBvalue b[]) {
   Screencoord x, y;
   Int32 *data, i;
-  I("writeRGB");
+  const char * MyName = "writeRGB";
+  I(MyName);
   if(!W->rgb) {
-    fprintf(stderr, "Ygl: writeRGB: not in RGB mode.\n");
+    Yprintf(MyName, "not in RGB mode.\n");
     exit(1);
   }
   x = X(W->xc);
   y = Y(W->yc);
   W->xc += n / W->xf;
   if(NULL == (data = (Int32*)malloc(n * sizeof(Int32)))) {
-    fprintf(stderr, "Ygl: readRGB: out of memory.\n");
+    Yprintf(MyName, "out of memory.\n");
     exit(1);
   }
   for(i = 0; i < n; i++) {
     data[i] = (b[i] << 16) + (g[i] << 8) + r[i];
   }
-  rect_write(x, y, x + n, y + 1, 4, data);  
+  rect_write(MyName, x, y, x + n, y + 1, 4, data);  
   free(data);
 }
