@@ -1,6 +1,6 @@
 /*
  *    Ygl: Run 2d-GL programs with standard X11 routines.
- *    (C) Fred Hucht 1993-95
+ *    (C) Fred Hucht 1993-96
  *    EMail: fred@thp.Uni-Duisburg.DE
  */
 
@@ -10,3 +10,11 @@
 #define COVERSLEEP	/* define to cover the system commands sleep
 			   and usleep with own versions that flushes
 			   Xlibs output buffer */ 
+
+/*#define RGBWIN*/
+
+#ifdef RGBWIN
+# define IF_RGBWIN(x) x
+#else
+# define IF_RGBWIN(x)
+#endif
