@@ -1,6 +1,6 @@
 /*
  *    Ygl: Run 2d-GL programs with standard X11 routines.
- *    (C) Fred Hucht 1993-95
+ *    (C) Fred Hucht 1993-96
  *    EMail: fred@thp.Uni-Duisburg.DE
  */
 
@@ -54,7 +54,18 @@ void circfi(Ico x, Ico y, Ico r)                   { I("circfi"); r=MAX(0,r); XF
 void circfs(Sco x, Sco y, Sco r)                   { I("circfs"); r=MAX(0,r); XFillArc(DWGC, X(x-r), Y(y+r), XR(2*r), YR(2*r), 0, 64*360); F;}
 
 /* Rects & Boxes */
-void rect  (Coo x1, Coo y1, Coo x2, Coo y2) { I("rect" ); XDrawRectangle(DWGC, X(MIN(x1,x2)), Y(MAX(y1,y2)), XR(ABS(x2-x1)), YR(ABS(y2-y1))); F;}
+void rect  (Coo x1, Coo y1, Coo x2, Coo y2) {
+  int x, y;
+  Uint w, h;
+  I("rect" );
+  x = X(MIN(x1,x2));
+  y = Y(MAX(y1,y2));
+  w = XR(ABS(x2-x1));
+  h = YR(ABS(y2-y1));
+  XDrawRectangle(DWGC, x, y, w, h);
+  F;
+}
+
 void sbox  (Coo x1, Coo y1, Coo x2, Coo y2) { I("sbox" ); XDrawRectangle(DWGC, X(MIN(x1,x2)), Y(MAX(y1,y2)), XR(ABS(x2-x1)), YR(ABS(y2-y1))); F;}
 void recti (Ico x1, Ico y1, Ico x2, Ico y2) { I("recti"); XDrawRectangle(DWGC, X(MIN(x1,x2)), Y(MAX(y1,y2)), XR(ABS(x2-x1)), YR(ABS(y2-y1))); F;}
 void sboxi (Ico x1, Ico y1, Ico x2, Ico y2) { I("sboxi"); XDrawRectangle(DWGC, X(MIN(x1,x2)), Y(MAX(y1,y2)), XR(ABS(x2-x1)), YR(ABS(y2-y1))); F;}
@@ -72,7 +83,7 @@ static Int32 PMode = Convex;
 static Ulong q_len = 0, q_max = 0;
 static XPoint *queue = NULL;
 
-static void q_add(int x, int y) {
+static void q_add(const char *caller, int x, int y) {
   if(q_max <= q_len) {
     q_max += 0x1000;
     if(queue == NULL)	/* Make new queue */
@@ -80,7 +91,7 @@ static void q_add(int x, int y) {
     else		/* Resize queue */
       queue = (XPoint*)realloc(queue, q_max * sizeof(XPoint));
     if(queue == NULL) {
-      fprintf(stderr, "Ygl: v: Out of memory.\n");
+      Yprintf(caller, "out of memory.\n");
       exit(1);
     }
   }
@@ -92,49 +103,49 @@ static void q_add(int x, int y) {
 
 void concave(Int32 bool) { PMode = bool ? Complex : Convex;}
 
-void  pmv2 (Coo x, Coo y) { SCP( =x,  =y); q_len = 0; q_add(X(x), Y(y));}
-void  pmv2i(Ico x, Ico y) { SCP( =x,  =y); q_len = 0; q_add(X(x), Y(y));}
-void  pmv2s(Sco x, Sco y) { SCP( =x,  =y); q_len = 0; q_add(X(x), Y(y));}
+void  pmv2 (Coo x, Coo y) { const char *MyName = "pmv2" ; I(MyName); SCP( =x,  =y); q_len = 0; q_add(MyName, X(x), Y(y));}
+void  pmv2i(Ico x, Ico y) { const char *MyName = "pmv2i"; I(MyName); SCP( =x,  =y); q_len = 0; q_add(MyName, X(x), Y(y));}
+void  pmv2s(Sco x, Sco y) { const char *MyName = "pmv2s"; I(MyName); SCP( =x,  =y); q_len = 0; q_add(MyName, X(x), Y(y));}
 
-void rpmv2 (Coo x, Coo y) { SCP(+=x, +=y); q_len = 0; q_add(X(x), Y(y));}
-void rpmv2i(Ico x, Ico y) { SCP(+=x, +=y); q_len = 0; q_add(X(x), Y(y));}
-void rpmv2s(Sco x, Sco y) { SCP(+=x, +=y); q_len = 0; q_add(X(x), Y(y));}
+void rpmv2 (Coo x, Coo y) { const char *MyName = "rpmv2" ; I(MyName); SCP(+=x, +=y); q_len = 0; q_add(MyName, X(x), Y(y));}
+void rpmv2i(Ico x, Ico y) { const char *MyName = "rpmv2i"; I(MyName); SCP(+=x, +=y); q_len = 0; q_add(MyName, X(x), Y(y));}
+void rpmv2s(Sco x, Sco y) { const char *MyName = "rpmv2s"; I(MyName); SCP(+=x, +=y); q_len = 0; q_add(MyName, X(x), Y(y));}
 
-void  pdr2 (Coo x, Coo y) { q_add(X(x), Y(y));}
-void  pdr2i(Ico x, Ico y) { q_add(X(x), Y(y));}
-void  pdr2s(Sco x, Sco y) { q_add(X(x), Y(y));}
+void  pdr2 (Coo x, Coo y) { const char *MyName = "pdr2" ; I(MyName); q_add(MyName, X(x), Y(y));}
+void  pdr2i(Ico x, Ico y) { const char *MyName = "pdr2i"; I(MyName); q_add(MyName, X(x), Y(y));}
+void  pdr2s(Sco x, Sco y) { const char *MyName = "pdr2s"; I(MyName); q_add(MyName, X(x), Y(y));}
 
-void rpdr2 (Coo x, Coo y) { q_add(queue[q_len-1].x + X(x), queue[q_len-1].y - Y(y));}
-void rpdr2i(Ico x, Ico y) { q_add(queue[q_len-1].x + X(x), queue[q_len-1].y - Y(y));}
-void rpdr2s(Sco x, Sco y) { q_add(queue[q_len-1].x + X(x), queue[q_len-1].y - Y(y));}
+void rpdr2 (Coo x, Coo y) { const char *MyName = "rpdr2" ; I(MyName); q_add(MyName, queue[q_len-1].x + X(x), queue[q_len-1].y - Y(y));}
+void rpdr2i(Ico x, Ico y) { const char *MyName = "rpdr2i"; I(MyName); q_add(MyName, queue[q_len-1].x + X(x), queue[q_len-1].y - Y(y));}
+void rpdr2s(Sco x, Sco y) { const char *MyName = "rpdr2s"; I(MyName); q_add(MyName, queue[q_len-1].x + X(x), queue[q_len-1].y - Y(y));}
 
-void pclos(void) { I("pclos"); XFillPolygon(DWGC, queue, q_len, PMode, CoordModeOrigin); F;}
+void pclos(void) { const char *MyName = "pclos";I(MyName); if(q_len) XFillPolygon(DWGC, queue, q_len, PMode, CoordModeOrigin); F;}
 
-#define MKP(n, p) { Int32 i; q_len = 0; for(i = 0; i < n; i++) { q_add(X(p[i][0]), Y(p[i][1]));}}
+#define MKP(n, p) { Int32 i; q_len = 0; for(i = 0; i < n; i++) { q_add(MyName, X(p[i][0]), Y(p[i][1]));}}
 
-void poly2 (Int32 n, Coo p[][2]) { I("poly2" ); MKP(n, p); q_add(XS(p[0][0]), YS(p[0][1])); XDrawLines(DWGC, queue, q_len, CoordModeOrigin); F;}
-void poly2i(Int32 n, Ico p[][2]) { I("poly2i"); MKP(n, p); q_add(XS(p[0][0]), YS(p[0][1])); XDrawLines(DWGC, queue, q_len, CoordModeOrigin); F;}
-void poly2s(Int32 n, Sco p[][2]) { I("poly2s"); MKP(n, p); q_add(XS(p[0][0]), YS(p[0][1])); XDrawLines(DWGC, queue, q_len, CoordModeOrigin); F;}
+void poly2 (Int32 n, Coo p[][2]) { const char *MyName = "poly2" ; I(MyName); MKP(n, p); q_add(MyName, XS(p[0][0]), YS(p[0][1])); XDrawLines(DWGC, queue, q_len, CoordModeOrigin); F;}
+void poly2i(Int32 n, Ico p[][2]) { const char *MyName = "poly2i"; I(MyName); MKP(n, p); q_add(MyName, XS(p[0][0]), YS(p[0][1])); XDrawLines(DWGC, queue, q_len, CoordModeOrigin); F;}
+void poly2s(Int32 n, Sco p[][2]) { const char *MyName = "poly2s"; I(MyName); MKP(n, p); q_add(MyName, XS(p[0][0]), YS(p[0][1])); XDrawLines(DWGC, queue, q_len, CoordModeOrigin); F;}
 
-void polf2 (Int32 n, Coo p[][2]) { I("polf2" ); MKP(n, p); SCP(=p[0][0],=p[0][1]); XFillPolygon(DWGC, queue, q_len, PMode, CoordModeOrigin); F;}
-void polf2i(Int32 n, Ico p[][2]) { I("polf2i"); MKP(n, p); SCP(=p[0][0],=p[0][1]); XFillPolygon(DWGC, queue, q_len, PMode, CoordModeOrigin); F;}
-void polf2s(Int32 n, Sco p[][2]) { I("polf2s"); MKP(n, p); SCP(=p[0][0],=p[0][1]); XFillPolygon(DWGC, queue, q_len, PMode, CoordModeOrigin); F;}
+void polf2 (Int32 n, Coo p[][2]) { const char *MyName = "polf2" ; I(MyName); MKP(n, p); SCP(=p[0][0],=p[0][1]); XFillPolygon(DWGC, queue, q_len, PMode, CoordModeOrigin); F;}
+void polf2i(Int32 n, Ico p[][2]) { const char *MyName = "polf2i"; I(MyName); MKP(n, p); SCP(=p[0][0],=p[0][1]); XFillPolygon(DWGC, queue, q_len, PMode, CoordModeOrigin); F;}
+void polf2s(Int32 n, Sco p[][2]) { const char *MyName = "polf2s"; I(MyName); MKP(n, p); SCP(=p[0][0],=p[0][1]); XFillPolygon(DWGC, queue, q_len, PMode, CoordModeOrigin); F;}
 
 /* Vertex graphics */
-void bgnpoint     (void) { I("bgnpoint"     ); W->vmode = VertexPoint; q_len = 0;}
-void bgnline      (void) { I("bgnline"      ); W->vmode = VertexLine;  q_len = 0;}
-void bgnclosedline(void) { I("bgnclosedline"); W->vmode = VertexCLine; q_len = 0;}
-void bgnpolygon   (void) { I("bgnpolygon"   ); W->vmode = VertexPoly;  q_len = 0;}
+void bgnpoint     (void) { const char *MyName = "bgnpoint";      I(MyName); W->vmode = VertexPoint; q_len = 0;}
+void bgnline      (void) { const char *MyName = "bgnline";       I(MyName); W->vmode = VertexLine;  q_len = 0;}
+void bgnclosedline(void) { const char *MyName = "bgnclosedline"; I(MyName); W->vmode = VertexCLine; q_len = 0;}
+void bgnpolygon   (void) { const char *MyName = "bgnpolygon";    I(MyName); W->vmode = VertexPoly;  q_len = 0;}
 
-void endpoint     (void) { if(W->vmode != VertexPoint) {fprintf(stderr, "Ygl: endpoint: missing bgnpoint\n"          ); exit(1);}                                XDrawPoints (DWGC, queue, q_len,        CoordModeOrigin); F;}
-void endline      (void) { if(W->vmode != VertexLine ) {fprintf(stderr, "Ygl: endline: missing bgnline\n"            ); exit(1);}                                XDrawLines  (DWGC, queue, q_len,        CoordModeOrigin); F;}
-void endclosedline(void) { if(W->vmode != VertexCLine) {fprintf(stderr, "Ygl: endclosedline: missing bgnclosedline\n"); exit(1);} q_add(queue[0].x, queue[0].y); XDrawLines  (DWGC, queue, q_len,        CoordModeOrigin); F;}
-void endpolygon   (void) { if(W->vmode != VertexPoly ) {fprintf(stderr, "Ygl: endpolygon: missing bgnpolygon\n"      ); exit(1);}                                XFillPolygon(DWGC, queue, q_len, PMode, CoordModeOrigin); F;}
+void endpoint     (void) { const char *MyName = "endpoint"     ; if(W->vmode != VertexPoint) {Yprintf(MyName, "missing bgnpoint\n"     ); exit(1);} if(q_len) {                                        XDrawPoints (DWGC, queue, q_len,        CoordModeOrigin); F;}}
+void endline      (void) { const char *MyName = "endline"      ; if(W->vmode != VertexLine ) {Yprintf(MyName, "missing bgnline\n"      ); exit(1);} if(q_len) {                                        XDrawLines  (DWGC, queue, q_len,        CoordModeOrigin); F;}}
+void endclosedline(void) { const char *MyName = "endclosedline"; if(W->vmode != VertexCLine) {Yprintf(MyName, "missing bgnclosedline\n"); exit(1);} if(q_len) { q_add(MyName, queue[0].x, queue[0].y); XDrawLines  (DWGC, queue, q_len,        CoordModeOrigin); F;}}
+void endpolygon   (void) { const char *MyName = "endpolygon"   ; if(W->vmode != VertexPoly ) {Yprintf(MyName, "missing bgnpolygon\n"   ); exit(1);} if(q_len) {                                        XFillPolygon(DWGC, queue, q_len, PMode, CoordModeOrigin); F;}}
 
-void v2s(Int16   v[2]) { if(W->vmode == VertexNone) { fprintf(stderr, "Ygl: v2s: Not in vertex mode\n"); exit(1);} q_add(X(v[0]), Y(v[1])); F;}
-void v2i(Int32   v[2]) { if(W->vmode == VertexNone) { fprintf(stderr, "Ygl: v2i: Not in vertex mode\n"); exit(1);} q_add(X(v[0]), Y(v[1])); F;}
-void v2f(Float32 v[2]) { if(W->vmode == VertexNone) { fprintf(stderr, "Ygl: v2f: Not in vertex mode\n"); exit(1);} q_add(X(v[0]), Y(v[1])); F;}
-void v2d(Float64 v[2]) { if(W->vmode == VertexNone) { fprintf(stderr, "Ygl: v2d: Not in vertex mode\n"); exit(1);} q_add(X(v[0]), Y(v[1])); F;}
+void v2s(Int16   v[2]) { const char *MyName = "v2s"; if(W->vmode == VertexNone) { Yprintf(MyName, "not in vertex mode\n"); exit(1);} q_add(MyName, X(v[0]), Y(v[1])); F;}
+void v2i(Int32   v[2]) { const char *MyName = "v2i"; if(W->vmode == VertexNone) { Yprintf(MyName, "not in vertex mode\n"); exit(1);} q_add(MyName, X(v[0]), Y(v[1])); F;}
+void v2f(Float32 v[2]) { const char *MyName = "v2f"; if(W->vmode == VertexNone) { Yprintf(MyName, "not in vertex mode\n"); exit(1);} q_add(MyName, X(v[0]), Y(v[1])); F;}
+void v2d(Float64 v[2]) { const char *MyName = "v2d"; if(W->vmode == VertexNone) { Yprintf(MyName, "not in vertex mode\n"); exit(1);} q_add(MyName, X(v[0]), Y(v[1])); F;}
 
 /* Text */
 void cmov2 (Coo x, Coo y) { W->xc = x; W->yc = y;}
@@ -142,11 +153,15 @@ void cmov2s(Sco x, Sco y) { W->xc = x; W->yc = y;}
 void cmov2i(Ico x, Ico y) { W->xc = x; W->yc = y;}
 
 void getcpos(Screencoord *x, Screencoord *y) {
-   Int32 xo, yo;
-  I("getcpos");
-  getorigin(&xo, &yo);  
-  *x = xo + X(W->xc);
-  *y = yo + W->ym - Y(W->yc);
+  /* Note: Due to a bug in GL getcpos() returns coordinates
+   * relative to the screen, not to the window. We emulate this. :-((
+   */
+  const char *MyName = "getcpos";
+  Int32 xo, yo;
+  I(MyName);
+  getorigin(&xo, &yo);
+  *x = xo + XR(W->xc);
+  *y = yo + YR(W->yc);
 #ifdef DEBUG
   fprintf(stderr, "getcpos: *x = %d, *y = %d\n", *x, *y);
 #endif
