@@ -1,5 +1,5 @@
-/* coltest.c by Fred Hucht (C) 1993-95.
-   Example for color animation with private colormap */
+/* coltest.c by Fred Hucht (C) 1993-96.
+ * Example for color animation with private colormap */
 
 #include <X11/Ygl.h>
 #include <stdio.h>
