@@ -1,5 +1,5 @@
-/* smile.c by Fred Hucht (C) 1993-95.
-   Short example program */
+/* smile.c by Fred Hucht (C) 1993-96.
+ * Short and simple example program */
 
 #include <X11/Ygl.h>
 #include <stdio.h>
